%% Create the 2moons data set.
% nn = 100; A = linspace(0,pi,nn);
% X1 = [cos(A);sin(A)]' + randn(nn,2)*0.1;
% X2 = [cos(-A)+1;sin(-A)+0.2]' + randn(nn,2)*0.1;
% X = [X1;X2];
load 2moons X

% In this example, we set K to 2 (the known number of clusters) and we set the
% target bandwidth to 0.1 (which gives a reasonably good density estimate). To
% illustrate the homotopy algorithm, we decrease the bandwidth s from large
% (s=1) to its target (s=0.1) in a geometric sequence of 20 values.

K = 2;						% Number of clusters
s = 10.^linspace(log10(1),log10(0.1),20);	% Bandwidth values
[CC,ZZ] = kmodes(X,K,s);			% Run kmodes
s = [Inf s];

% Plot results
ax = [-1.3 2.2 -1.1 1.2];
figure(1); plot(X(:,1),X(:,2),'bo'); daspect([1 1 1]); axis(ax);
figure(2); COLORSX = {'r+','b+','g+'}; N = size(X,1);
for its=1:length(CC)
  C = CC{its}; IDX = ZZ{its}; Z = zeros(N,K); Z(sub2ind([N,K],(1:N)',IDX)) = 1;
  
  clf; hold on;
  for k=1:K
    idx = find(IDX==k); plot(X(idx,1),X(idx,2),COLORSX{k},'MarkerSize',8);
  end
  plot(C(:,1),C(:,2),'ko','MarkerSize',16,'MarkerFaceColor','k');
  
  % Plot path
  plot(CC{1}(:,1),CC{1}(:,2),'k*','MarkerSize',16,'MarkerFaceColor','k');
  for i=2:its
    p1=[CC{i-1}(:,1), CC{i}(:,1), NaN(K,1)]'; p1=p1(:);
    p2=[CC{i-1}(:,2), CC{i}(:,2), NaN(K,1)]'; p2=p2(:);
    plot(p1,p2,'k','LineWidth',2);
  end

  % Plot normalized KDE for each cluster.
  if its>1
    for j=1:K
      t1 = linspace(ax(1),ax(2),50); t2 = linspace(ax(3),ax(4),50);
      [Y1,Y2] = meshgrid(t1,t2); Y = [Y1(:),Y2(:)];
      pdf = exp(-sqdist(Y,X)/(2*s(its)^2))*Z(:,j)/sum(Z(:,j))/...
            (sqrt(2*pi)*s(its))^2;
      contour(Y1,Y2,reshape(pdf,50,50),10,COLORSX{j}(1));
    end
  end
  daspect([1 1 1]); axis(ax); box on; title(['\sigma=',num2str(s(its))]);
  pause
end

